==> ecomero-erpcore-1.0.42.zip: Service/ImageService.php <==
<?php declare(strict_types=1);
/**
 *
 *           a88888P8
 *          d8'
 * .d8888b. 88        .d8888b. 88d8b.d8b. .d8888b. .dd888b. .d8888b.
 * 88ooood8 88        88'  `88 88'`88'`88 88ooood8 88'    ` 88'  `88
 * 88.  ... Y8.       88.  .88 88  88  88 88.  ... 88       88.  .88
 * `8888P'   Y88888P8 `88888P' dP  dP  dP `8888P'  dP       `88888P'
 *
 *           Copyright © eComero Management AB, All rights reserved.
 *
 */
namespace Ecomero\ErpCore\Service;

use Ecomero\ErpCore\Helper\ErpLogger;
use Ecomero\ErpCore\Helper\Settings;
use Ecomero\ErpCore\Model\Capability;
use Ecomero\ErpCore\Model\ErpAdapterInterface;

use Magento\Catalog\Model\Product\Image\UrlBuilder;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory;
use Magento\Framework\App\ObjectManager;

use Symfony\Component\Console\Output\OutputInterface;

class ImageService
{
    protected $logger;
    protected $erp;
    protected $settings;
    protected $productService;
    protected $productCollectionFactory;
    protected $mediaUrl;
    protected $imageUrlBuilder;
    protected $configurableFactory;
    protected $productFactory;

    public function __construct(
        UrlBuilder $urlBuilder = null,
        ConfigurableFactory $configurableFactory,
        ProductFactory $productFactory,
        ErpLogger $logger,
        ErpAdapterInterface $erp,
        Settings $settings,
        ProductService $productService,
        CollectionFactory $productCollectionFactory
    ) {
        $this->imageUrlBuilder = $urlBuilder ?? ObjectManager::getInstance()->get(UrlBuilder::class);
        $this->configurableFactory = $configurableFactory;
        $this->productFactory = $productFactory;
        $this->logger = $logger;
        $this->erp = $erp;
        $this->settings = $settings;
        $this->productService = $productService;
        $this->productCollectionFactory = $productCollectionFactory;
    }

    public function exportCommandLine(OutputInterface $output, bool $forceUnlock) : void
    {
        $this->logger->info('Starting product image export from command line');
        $this->logger->setOutput($output);
        $this->export($forceUnlock);
    }

    public function exportWeb(bool $forceUnlock = false) : void
    {
        $this->logger->info('Starting product image export from web');
        $this->export($forceUnlock);
    }

    private function getImageUrl($product) : string
    {
        $imageUrl = $this->getImageUrlForType($product, 'style_image');
        if ($imageUrl === '') {
            $imageUrl = $this->getImageUrlForType($product, 'small_image');
        }
        return $imageUrl;
    }

    private function getImageUrlForType($product, string $imageType) : string
    {
        $imageUrl = '';

        foreach ($product->getMediaGalleryImages() as $mediaGalleryImage) {
            $imageFile = $mediaGalleryImage->getFile();

            if ($product->getData($imageType) === $imageFile) {
                $imageUrl = $this->imageUrlBuilder->getUrl(
                    $imageFile,
                    'pagebuilder_product_image_admin'
                );
                break;
            }
        }
        return $imageUrl;
    }

    private function export(bool $forceUnlock = false) : void
    {
        if ($forceUnlock) {
            $this->settings->releaseCronLock();
        }

        if ($this->settings->requestCronLock($this->logger) == false) {
            return;
        }

        $capabilities = $this->erp->getCapabilities();
        foreach ($capabilities as $capability) {
            if (!$capability->isCapabilitySupported(Capability::PRODUCT_PICTURE_EXPORT)) {
                $this->logger->warning('Export of product images is not supported by the ' . $capability->getName() . ' integration');
                $this->settings->releaseCronLock();
                return;
            }
        }

        try {
            $this->logger->debug('Exporting images');

            $productCollection = $this->productCollectionFactory->create();
            $productCollection->addAttributeToSelect('*');
            $productCollection->addAttributeToFilter('erp_id', ['like' => '%-%-%-%-%']);

            foreach ($productCollection as $product) {
                $bcAttrib = $product->getCustomAttribute('erp_id');
                $itemId = $bcAttrib->getValue();

                $imageUrl = $this->getImageUrl($product);
                if ($imageUrl === '') {
                    // Fallback, check image on parent product
                    $configProduct = $this->configurableFactory->create();
                    $parentArray = $configProduct->getParentIdsByChild($product->getId());
                    if (isset($parentArray[0])) {
                        $parentProduct = $this->productFactory->create()->load($parentArray[0]);
                        $imageUrl = $this->getImageUrl($parentProduct);
                    }
                }

                $rc = $this->erp->addPicture($itemId, $imageUrl);
                if ($rc) {
                    $this->logger->debug('  - Exporting image ' . $imageUrl .
                                         ' for "' . $product->getName() .
                                         '" (' . $itemId . ') status code ' . $rc);
                } else {
                    $this->logger->debug('  - Failed exporting image ' . $imageUrl .
                                         ' for "' . $product->getName() .
                                         '" (' . $itemId . ')');
                }
            }
        } catch (\Zend\Http\Exception\RuntimeException $exception) {
            $this->logger->error($exception->getMessage());
        } catch (\Magento\Framework\Exception\LocalizedException $exception) {
            $this->logger->error($exception->getMessage());
        }
        $this->logger->info('Export completed');
        $this->settings->releaseCronLock();
    }
}

==> ecomero-erpcore-1.2.1.zip: Service/ImageService.php <==
<?php declare(strict_types=1);
/**
 *
 *           a88888P8
 *          d8'
 * .d8888b. 88        .d8888b. 88d8b.d8b. .d8888b. .dd888b. .d8888b.
 * 88ooood8 88        88'  `88 88'`88'`88 88ooood8 88'    ` 88'  `88
 * 88.  ... Y8.       88.  .88 88  88  88 88.  ... 88       88.  .88
 * `8888P'   Y88888P8 `88888P' dP  dP  dP `8888P'  dP       `88888P'
 *
 *           Copyright © eComero Management AB, All rights reserved.
 *
 */
namespace Ecomero\ErpCore\Service;

use Ecomero\ErpCore\Helper\ErpLogger;
use Ecomero\ErpCore\Helper\Settings;
use Ecomero\ErpCore\Model\Capability;
use Ecomero\ErpCore\Model\ErpAdapterInterface;

use Magento\Catalog\Model\Product\Image\UrlBuilder;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory;
use Magento\Framework\App\ObjectManager;

use Symfony\Component\Console\Output\OutputInterface;

class ImageService
{
    protected $logger;
    protected $erp;
    protected $settings;
    protected $productService;
    protected $productCollectionFactory;
    protected $mediaUrl;
    protected $imageUrlBuilder;
    protected $configurableFactory;
    protected $productFactory;

    public function __construct(
        UrlBuilder $urlBuilder = null,
        ConfigurableFactory $configurableFactory,
        ProductFactory $productFactory,
        ErpLogger $logger,
        ErpAdapterInterface $erp,
        Settings $settings,
        ProductService $productService,
        CollectionFactory $productCollectionFactory
    ) {
        $this->imageUrlBuilder = $urlBuilder ?? ObjectManager::getInstance()->get(UrlBuilder::class);
        $this->configurableFactory = $configurableFactory;
        $this->productFactory = $productFactory;
        $this->logger = $logger;
        $this->erp = $erp;
        $this->settings = $settings;
        $this->productService = $productService;
        $this->productCollectionFactory = $productCollectionFactory;
    }

    public function exportCommandLine(OutputInterface $output, bool $forceUnlock) : void
    {
        $this->logger->info('Starting product image export from command line');
        $this->logger->setOutput($output);
        $this->export($forceUnlock);
    }

    public function exportWeb(bool $forceUnlock = false) : void
    {
        $this->logger->info('Starting product image export from web');
        $this->export($forceUnlock);
    }

    private function getImageUrl($product) : string
    {
        $imageUrl = $this->getImageUrlForType($product, 'style_image');
        if ($imageUrl === '') {
            $imageUrl = $this->getImageUrlForType($product, 'small_image');
        }
        return $imageUrl;
    }

    private function getImageUrlForType($product, string $imageType) : string
    {
        $imageUrl = '';

        foreach ($product->getMediaGalleryImages() as $mediaGalleryImage) {
            $imageFile = $mediaGalleryImage->getFile();

            if ($product->getData($imageType) === $imageFile) {
                $imageUrl = $this->imageUrlBuilder->getUrl(
                    $imageFile,
                    'pagebuilder_product_image_admin'
                );
                break;
            }
        }
        return $imageUrl;
    }

    private function export(bool $forceUnlock = false) : void
    {
        if ($forceUnlock) {
            $this->settings->releaseCronLock();
        }

        if ($this->settings->requestCronLock($this->logger) == false) {
            return;
        }

        $capabilities = $this->erp->getCapabilities();
        foreach ($capabilities as $capability) {
            if (!$capability->isCapabilitySupported(Capability::PRODUCT_PICTURE_EXPORT)) {
                $this->logger->warning('Export of product images is not supported by the ' . $capability->getName() . ' integration');
                $this->settings->releaseCronLock();
                return;
            }
        }

        try {
            $this->logger->debug('Exporting images');

            $productCollection = $this->productCollectionFactory->create();
            $productCollection->addAttributeToSelect('*');
            $productCollection->addAttributeToFilter('erp_id', ['like' => '%-%-%-%-%']);

            foreach ($productCollection as $product) {
                $this->processProduct($product);
            }
        } catch (\Throwable $exception) {
            $this->logger->error($exception->getMessage());
        }
        
        $this->logger->info('Export completed');
        $this->settings->releaseCronLock();
    }

    public function processProduct($product)
    {
        try {
            $bcAttrib = $product->getCustomAttribute('erp_id');
            $itemId = $bcAttrib->getValue();
    
            $imageUrl = $this->getImageUrl($product);
            if ($imageUrl === '') {
                // Fallback, check image on parent product
                $configProduct = $this->configurableFactory->create();
                $parentArray = $configProduct->getParentIdsByChild($product->getId());
                if (isset($parentArray[0])) {
                    $parentProduct = $this->productFactory->create()->load($parentArray[0]);
                    $imageUrl = $this->getImageUrl($parentProduct);
                }
            }
    
            $rc = $this->erp->addPicture($itemId, $imageUrl);
            if ($rc) {
                $this->logger->debug('  - Exporting image ' . $imageUrl .
                                     ' for "' . $product->getName() .
                                     '" (' . $itemId . ') status code ' . $rc);
            } else {
                $this->logger->debug('  - Failed exporting image ' . $imageUrl .
                                     ' for "' . $product->getName() .
                                     '" (' . $itemId . ')');
            }
        } catch (\Zend\Http\Exception\RuntimeException $exception) {
            $this->logException($product, $exception->getMessage());
        } catch (\Magento\Framework\Exception\LocalizedException $exception) {
            $this->logException($product, $exception->getMessage());
        }
    }

    protected function logException($product, $exception) : self
    {
        $this->logger->error($exception->getMessage());
        return $this;
    }
}
